extern crate minifb;

use std::error::Error;
use minifb::{Window, WindowOptions, Key};

const WIDTH: usize = 640;
const HEIGHT: usize = 480;

trait Shader {
	fn shade(&mut self, x: usize, y: usize) -> u32;
}

struct TestShader {
	noise: u32,
	carry: u32,
	pub seed: u32,
}
impl TestShader {
	pub fn new(seed: u32) -> TestShader {
		TestShader {
			noise: 0,
			carry: 0,
			seed
		}
	}
}
impl Shader for TestShader {
	fn shade(&mut self, x_: usize, y_: usize) -> u32 {
		self.noise = self.seed;
		self.noise >>= 3;
		self.noise ^= self.seed;
		self.carry = self.noise & 1;
		self.noise >>= 1;
		self.seed >>=1;
		self.seed |= (self.carry << 30);
		self.noise &= 0xff;
		
		return (self.noise << 16) | (self.noise << 8) | self.noise;
	}
}

fn run() -> Result<(), Box<Error>> {
	let mut buffer: Vec<u32> = vec![0; WIDTH * HEIGHT];
	let mut window = Window::new("minifb",
		WIDTH,
		HEIGHT,
		WindowOptions::default())?;

	let mut shader = TestShader::new(0xcafe);

	while window.is_open() && !window.is_key_down(Key::Escape) {
		for x in 0..WIDTH {
			for y in 0..HEIGHT {
				buffer[y*WIDTH + x] = shader.shade(x, y);
			}
		}

		window.update_with_buffer(&buffer);
	}

	Ok(())
}

fn main() {
	if let Err(e) = run() {
		println!("Error: {}", e);
	} 
}